@@ -15,12 +15,14 @@ from django.contrib.contenttypes.models import ContentType
 from django.core.urlresolvers import reverse
 from django.core.validators import RegexValidator
 from django.db import models
+from django.template.defaultfilters import slugify
 from django.utils.timezone import timedelta
 from django.utils.translation import ugettext_lazy as _
 
 from calendarium.constants import FREQUENCY_CHOICES, OCCURRENCE_DECISIONS
 from calendarium.utils import OccurrenceReplacer
 from calendarium.widgets import ColorPickerWidget
+from filer.fields.image import FilerImageField
 from south.modelsinspector import add_introspection_rules
 
 
@@ -119,6 +121,7 @@ class Event(EventModelMixin):
     :rule: FK to the definition of the recurrence of an event.
     :end_recurring_period: The possible end of the recurring definition.
     :title: The title of the event.
+    :image: Optional image of the event.
 
     """
 
@@ -152,6 +155,11 @@ class Event(EventModelMixin):
         verbose_name=_('Title'),
     )
 
+    image = FilerImageField(
+        verbose_name=_('Image'),
+        null=True, blank=True,
+    )
+
     objects = EventModelManager()
 
     def get_absolute_url(self):
@@ -225,9 +233,6 @@ class Event(EventModelMixin):
                 # if the persistent occ falls into the period, replace it
                 if (end and p_occ.start < end) and p_occ.end >= start:
                     estimated_occ = p_occ
-                else:
-                    occ = occurrence_gen.next()
-                    continue
             else:
                 # if there is no persistent match, use the original occ
                 estimated_occ = occ
@@ -250,19 +255,45 @@ class Event(EventModelMixin):
 
 
 class EventCategory(models.Model):
-    """The category of an event."""
+    """
+    The category of an event.
+
+    :name: The name of the category.
+    :slug: The slug of the category.
+    :color: The color of the category.
+    :parent: Allows you to create hierarchies of event categories.
+
+    """
     name = models.CharField(
         max_length=256,
         verbose_name=_('Name'),
     )
 
+    slug = models.SlugField(
+        max_length=256,
+        verbose_name=_('Slug'),
+        blank=True,
+    )
+
     color = ColorField(
         verbose_name=_('Color'),
     )
 
+    parent = models.ForeignKey(
+        'calendarium.EventCategory',
+        verbose_name=_('Parent'),
+        related_name='parents',
+        null=True, blank=True,
+    )
+
     def __unicode__(self):
         return self.name
 
+    def save(self, *args, **kwargs):
+        if not self.slug:
+            self.slug = slugify(self.name)
+        return super(EventCategory, self).save(*args, **kwargs)
+
 
 class EventRelation(models.Model):
     """
@@ -359,7 +390,7 @@ class Occurrence(EventModelMixin):
         is_only = False
         gen = self.event.get_occurrences(
             self.start, self.event.end_recurring_period)
-        occs = [occ for occ in gen]
+        occs = list(set([occ for occ in gen]))
         if len(occs) == 1:
             is_only = True
         elif len(occs) > 1 and self == occs[-1]:
