@@ -12,6 +12,7 @@ from django.views.generic import (
     CreateView,
     DeleteView,
     DetailView,
+    ListView,
     RedirectView,
     TemplateView,
     UpdateView,
@@ -242,7 +243,7 @@ class OccurrenceViewMixin(object):
         day = int(kwargs.get('day'))
         try:
             date = datetime(year, month, day, tzinfo=utc)
-        except TypeError:
+        except (TypeError, ValueError):
             raise Http404
         # this should retrieve the one single occurrence, that has a
         # matching start date
@@ -258,16 +259,16 @@ class OccurrenceViewMixin(object):
             self.occurrence = occ
         else:
             raise Http404
-        self.object = self.occurrence
+        self.object = occ
         return super(OccurrenceViewMixin, self).dispatch(
             request, *args, **kwargs)
 
     def get_object(self):
-        return self.occurrence
+        return self.object
 
     def get_form_kwargs(self):
         kwargs = super(OccurrenceViewMixin, self).get_form_kwargs()
-        kwargs.update({'initial': model_to_dict(self.occurrence)})
+        kwargs.update({'initial': model_to_dict(self.object)})
         return kwargs
 
 
@@ -299,3 +300,39 @@ class OccurrenceDetailView(OccurrenceViewMixin, DetailView):
 class OccurrenceUpdateView(OccurrenceViewMixin, UpdateView):
     """View to edit an occurrence of an event."""
     pass
+
+
+class UpcomingEventsAjaxView(CategoryMixin, ListView):
+    template_name = 'calendarium/partials/upcoming_events.html'
+    context_object_name = 'occurrences'
+
+    def dispatch(self, request, *args, **kwargs):
+        if request.GET.get('category'):
+            self.category = EventCategory.objects.get(
+                slug=request.GET.get('category'))
+        else:
+            self.category = None
+        if request.GET.get('count'):
+            self.count = int(request.GET.get('count'))
+        else:
+            self.count = None
+        return super(UpcomingEventsAjaxView, self).dispatch(
+            request, *args, **kwargs)
+
+    def get_context_data(self, **kwargs):
+        ctx = super(UpcomingEventsAjaxView, self).get_context_data(**kwargs)
+        ctx.update(self.get_category_context(**kwargs))
+        ctx.update({'show_excerpt': True, })
+        return ctx
+
+    def get_queryset(self):
+        qs_kwargs = {
+            'start': now(),
+            'end': now() + timedelta(365),
+        }
+        if self.category:
+            qs_kwargs.update({'category': self.category, })
+        qs = Event.objects.get_occurrences(**qs_kwargs)
+        if self.count:
+            return qs[:self.count]
+        return qs
